--------------------------------------------------------------------------------
-- Copyright  2019								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- For more information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local AddonName, NS = ...

local L
local Container

local clip, find, sub = string.trim, string.find, string.sub

local DefaultSkinCount = 10
local FrameWidth, FrameHeight = 410, 345
local PrimaryTray, TraySettings
local CurrentButton = {}
local MailFrame = MiniMapMailFrame or MinimapCluster.MailFrame or MailFrame

local function ConfigureUI(parent)
	L = NS.TEXTS
	if Container then
		Container:SetShown(not Container:IsShown())
		return Container
	end
	PrimaryTray = NS:GetFrame("Utility", "ButtonTray1")
	TraySettings = {}
	local function LoadTraySettings()
		table.wipe(TraySettings)
		NS.Lib:CopyTable(NS:GetSettings("UI").buttontrays, TraySettings)
	end
	LoadTraySettings()
	local ButtonSettings = {}
	local function SetButton(id, text, icon)
		ButtonSettings.id = id
		ButtonSettings.text = text
		ButtonSettings.icon = icon
		ButtonSettings.nolocale = false
		ButtonSettings.tip = nil
	end
	local ObjectExclude = {
		[GameTimeFrame]=true,
		[MailFrame]=true,
		[Minimap]=true,
	}
	if GarrisonLandingPageMinimapButton then
		ObjectExclude[GarrisonLandingPageMinimapButton]=true
	end
	if MiniMapTracking then
		ObjectExclude[MiniMapTracking]=true
	end
	if MiniMapTrackingButton then
		ObjectExclude[MiniMapTrackingButton]=true
	end
	if QueueStatusMinimapButton then
		ObjectExclude[QueueStatusMinimapButton]=true
	end
	local QSB = Queuestatusbutton or QueueStatusButton
	if QSB then
		ObjectExclude[QSB]=true
	end
	local MiniButtonExcludeList = {
		"ActionButton=true",
		"MultiBar=true",
		"MainMenu=true",
		"FauxMazzle",
		"AutoBar",
		"Trinket",
	}
 
	local MMPortraits = { 
		{ text="3D", value=0 }, 
		{ text="2D", value=1 }, 
		{ text="FM", value=2 }, 
	}

	local OnLeftClick = {
		{ text=L.Options, value=1 }, 
		{ text=L.Menu, value=2 }, 
	}

	local GoldValues = { 
		{ text=L.Never, value=nil }, 
	        { text="1,000", value=9999999 }, --1,000g
	        { text="10,000", value=99999999 }, --10,000g
	        { text="100,000", value=999999999 }, --100,000g
	        { text="1,000,000", value=9999999999 }, --1,000,000g
	        { text="10,000,000", value=99999999999 }, --1,000,000g
	}
	local function RandomTip(self)
		if self:GetChecked() then
			return L.GetText(L.SkinsRandomExcludeTip, L.Excluded)
		else
			return L.GetText(L.SkinsRandomExcludeTip, L.Included)
		end
	end
	local ExcludeText = ""
	local function SkinsButtonCreate(self) -- self = button base
--		self:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
		self:SetHighlightTexture("Interface\\Common\\bluemenu-main")
		self:GetHighlightTexture():SetTexCoord(0.00390625, 0.87890625, 0.75195313, 0.83007813)
		self.Text = NS.Lib:CreateFont("Text", self)
		self.Checked = self:CreateTexture(nil, "ARTWORK", nil, 2)
		
		self.Checked:SetSize(8, 8)
		self.Checked:SetPoint("RIGHT", self.Text, "LEFT", -2, 0)
		self.Checked:Hide()
		self.Checked:SetTexture(NS.TexturePath.."MarkerWhite")
		self.Checked:SetVertexColor(0.9, 0.4, 0.1)
		self.ExcludeLED = NS.Lib:CreateLedCheckButton("Exclude", self, {}, "x", "Red", "Green")
		self.ExcludeLED:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_TOP", 0, 4)
--			GameTooltip:SetText(RandomTip(self))
			GameTooltip:SetText(RandomTip(self), nil, nil, nil, nil, true)
		end)
		self.ExcludeLED:SetScript("OnLeave", function(self)
			GameTooltip:Hide()
		end)
--		self.ExcludeLED:SetToolTip(ExcludeText, "ANCHOR_TOP")
		self.ExcludeLED:SetEnabled(false)
		self.ExcludeLED:SetSize(15, 15)
		self.ExcludeLED:SetPoint("RIGHT")
		self.Text:SetPoint("TOPLEFT", self, 8, 0)
		self.Text:SetPoint("BOTTOMRIGHT", self.ExcludeLED, "BOTTOMLEFT", -2, 0)
			
		self.ExcludeLED:HookScript("OnClick", function(self)
			local parent = self:GetParent()
			local grandParent = parent:GetParent()
			grandParent.Data.ignore[grandParent.Table[parent.index].value] = self:GetChecked()
			self:GetScript("OnEnter")(self)
		end)
		function self:LED_SetEnabled(enabled)
			self.ExcludeLED:SetEnabled(enabled)
		end
	end

	local function SkinsButtonUpdate(self, table, index, data, savedvalue)
		local tablevalue = table[index].value
--		local parent = self:GetParent()
		if tablevalue == savedvalue then
			self.Checked:Show()
		else
			self.Checked:Hide()
		end
		if data.ignore[tablevalue] then
			self.ExcludeLED:SetChecked(true)
		else
			self.ExcludeLED:SetChecked(false)
		end
	end
	
	local CheckButtons = {}
	local Controls = {}
	local function CopySettings(character, charsettings)
		NS:CopySettings(character, charsettings, "UI", Controls, CheckButtons)
		local Settings = NS:GetSettings("UI").skin.ignore
		SrcSettings = NS:CharacterSettings("skin", charsettings, "UI").ignore
		table.wipe(Settings)
		NS.Lib:CopyTable(SrcSettings, Settings)
		Container.Skins:Init()
		Container.Skins:LED_UpdateEnabled(NS:GetSettings("UI").skin.random)
		
		Settings = NS:GetSettings("UI").buttontrays
		for tray=1, #Settings do
			for point, buttonlist in pairs(Settings[tray]) do
				for list=#buttonlist, 1, -1 do
					if list > 1 then
						tremove(buttonlist, list)
					else
						for button=#buttonlist[list], 1, -1 do
							if not buttonlist[button].system then
								tremove(buttonlist, buttonlist[button])
							end
						end
					end
				end
			end
		end
		SrcSettings = NS:CharacterSettings(true, charsettings, "UI").buttontrays
		NS.Lib:CopyTable(SrcSettings, Settings, NS.Lib.SaveTables + NS.Lib.SaveKeys)
		NS:GetOption("ManageMiniButtons")()
	end
--	local function ResetSettings()
--		if not next(SaveUI) then
--			NS.DebugPrint("No saved settings to restore!")
--			return
--		end
--	end
----------------------------------------------------------------------------------------------------------------
	local SkinList = NS:GetOption("GetAccessoryList")("Skin")
	local DefaultSkin = NS:GetOption("GetDefaultAccessory")("Skin")
	local Skins = {}
	
	for k, v in pairs(SkinList) do
		if k ~= DefaultSkin then
			tinsert(Skins, { title=v.title..v.author, value=k, })
		end
	end
	NS:GetOption("SortTable")(Skins, "title")
	local DefaultOptions = NS:GetOption("GetAccessoryInfo")(DefaultSkin, "Skin")
	tinsert(Skins, 1, { title=DefaultOptions.title..DefaultOptions.author, value=DefaultSkin, })
	
	local Settings = NS:GetSettings("UI")
	
	Container = CreateFrame("Frame", "$parentUI", parent)
--	Container:Hide()
	Container:SetSize(FrameWidth, FrameHeight)
--	Container:SetFrameStrata("BACKGROUND")
--	Container:SetFrameStrata("LOW")
--	Container:SetFrameLevel(2)
	
	Container.SkinsLabel = NS:CreateLabel(Container, "UISkins", L.Skins)
	Container.SkinsLabel:SetPoint("TOP", -125, -10)

----------- Skins

	Container.SkinRandom = NS.Lib:CreateLedCheckButton("SkinsRandom", Container, Settings.skin, "random", "Green")
	Container.SkinRandom:SetSize(20, 20)
	NS:AddCheckLabel(Container.SkinRandom, L.Random, "RIGHT")
	Container.SkinRandom:SetToolTip(L.SkinsRandomTip)
	
	function Container.SkinRandom.PostUpdate(self)
		self:GetParent().SkinDaily:SetEnabled(self:GetChecked())
		self:GetParent().Skins:LED_UpdateEnabled(self:GetChecked())
	end
	CheckButtons[Container.SkinRandom] = "skin"
	
	Container.SkinDaily = NS.Lib:CreateLedCheckButton("SkinsDaily", Container.SkinRandom, Settings.skin, "daily", "Green")
	Container.SkinDaily:SetSize(20, 20)
	NS:AddCheckLabel(Container.SkinDaily, L.Daily, "LEFT")
	Container.SkinDaily:SetToolTip(L.SkinsDailyTip)
	CheckButtons[Container.SkinDaily] = "skin"

	Container.Skins = NS.Lib:CreateScrollList("Skins", Container, Settings.skin, "skin", Skins, "title", "value", DefaultSkinCount, SkinsButtonUpdate, SkinsButtonCreate)
	Container.Skins:SetSize(140, DefaultSkinCount * 20)
	Container.Skins:SetPoint("TOPLEFT", Container, 5, -55)
	function Container.Skins:LED_UpdateEnabled(enabled)
		for i=1, Container.Skins:NumRows() do
			Container.Skins:GetRow(i):LED_SetEnabled(enabled)
		end
	end
	function Container.Skins:PostUpdate()
		NS:GetOption("LoadSkin")(true)
	end
	Container.SkinRandom:PostUpdate()

	Container.SkinRandom:SetPoint("BOTTOMLEFT", Container.Skins, "TOPLEFT", 4, 10)
	Container.SkinDaily:SetPoint("BOTTOMRIGHT", Container.Skins, "TOPRIGHT", 20, 10)

--------- Combat Colours
	Container.ColorsLabel = NS:CreateLabel(Container, "UIBGColors", L.GetText(L.BackgroundColors, L.Background, L.Colors))
	Container.ColorsLabel:SetPoint("TOP", 110, -10)

	local function SetCombatColorEnable(self)
		local parent = self:GetParent()
		local checked = self:GetChecked()
		parent.CombatColor:SetEnabled(checked)
		parent.NoCombatColor:SetEnabled(checked)
	end
	Container.CombatColorEnable = NS.Lib:CreateLedCheckButton("CombatColorEnable", Container, Settings.skin, "color", "Green")
	Container.CombatColorEnable:SetSize(20, 20)
	Container.CombatColorEnable:SetPoint("TOP", Container, 30, -25)
	NS:AddCheckLabel(Container.CombatColorEnable, L.Enable, "RIGHT")
	Container.CombatColorEnable:SetToolTip(L.CombatColorsTip)
	Container.CombatColorEnable.PostUpdate = NS:GetOption("CombatColorsInit")
	Container.CombatColorEnable.InitColors = NS:GetOption("CombatColorsInit")
	Container.CombatColorEnable.UpdateColors = NS:GetOption("CombatColorsApply")
	function Container.CombatColorEnable:TextCombatColors(nocombat)
		self:InitColors()
	end
	Container.CombatColorEnable:SetScript("OnHide", function(self)
		self:TextCombatColors(true)
	end)
	Container.CombatColorEnable:HookScript("OnClick", SetCombatColorEnable)
	CheckButtons[Container.CombatColorEnable] = "skin"
	
	Container.CombatColor = NS.Lib:CreateLEDColorPickerButton("CombatColor", Container, Settings.skin, "colorcombat")
	Container.CombatColor:SetSize(18, 18)
	Container.CombatColor:SetPoint("TOPRIGHT", Container, "TOPRIGHT", -10, -25)
	Container.CombatColor.DefaultColor = NS:GetOption("CombatColors").CombatDefault
	Container.CombatColor:SetToolTip(L.ColorPickerTip)
	Container.CombatColor.PostUpdate = function()
		Container.CombatColorEnable:TextCombatColors()
	end
	Controls[Container.CombatColor] = "skin"
	
	Container.NoCombatColor = NS.Lib:CreateLEDColorPickerButton("NoCombatColor", Container, Settings.skin, "colornotcombat")
	Container.NoCombatColor:SetSize(18, 18)
	Container.NoCombatColor:SetPoint("RIGHT", Container.CombatColor, "LEFT", -60, 0)
	Container.NoCombatColor:SetLabel("- "..L.Combat.." +", "RIGHT")
	Container.NoCombatColor:GetLabel():SetPoint("RIGHT", Container.CombatColor, "LEFT", -5, 0)
	Container.NoCombatColor.DefaultColor = NS:GetOption("CombatColors").NotCombatDefault
	Container.NoCombatColor:SetToolTip(L.ColorPickerTip)
	Container.NoCombatColor.PostUpdate = function()
		Container.CombatColorEnable:TextCombatColors(true)
	end
	SetCombatColorEnable(Container.CombatColorEnable)
	Controls[Container.NoCombatColor] = "skin"

------- Misc. Options
	Container.MiscellaneousLabel = NS:CreateLabel(Container, "UIMiscellaneous", L.Miscellaneous)
	Container.MiscellaneousLabel:SetPoint("TOP", Container.NoCombatColor, 0, -30)
--[[
	Container.LookCursor = NS.Lib:CreateLedCheckButton("MouseLookCursor", Container, Settings, "mouselookcursor", "Green")
	Container.LookCursor:SetPoint("TOPLEFT", Container.CombatColorEnable, "BOTTOMLEFT", 10, -21)
	NS:AddCheckLabel(Container.LookCursor, L.MouseLookCursor, "RIGHT", 15, 0)
	Container.LookCursor:SetToolTip(L.MouseLookCursorTip)
	Container.LookCursor.PostUpdate = function(self)
		NS:GetOption("ConfirmDelete")()
	end
	CheckButtons[Container.LookCursor] = true -- true = use base settings, not branch
]]--

	Container.ConfirmDelete = NS.Lib:CreateLedCheckButton("ConfirmDelete", Container, Settings, "confirmdelete", "Green")
	Container.ConfirmDelete:SetPoint("TOPLEFT", Container.CombatColorEnable, "BOTTOMLEFT", 10, -21)
	NS:AddCheckLabel(Container.ConfirmDelete, L.ConfirmDelete, "RIGHT", 15, 0)
	Container.ConfirmDelete:SetToolTip(L.ConfirmDeleteTip)
	Container.ConfirmDelete.PostUpdate = function(self)
		NS:GetOption("ConfirmDelete")()
	end
	CheckButtons[Container.ConfirmDelete] = true -- true = use base settings, not branch

	if NS:IsValidVersion(WOW_PROJECT_MAINLINE) or NS:IsValidVersion(WOW_PROJECT_WRATH_CLASSIC) then
		Container.QuestTrackerCollapse = NS.Lib:CreateLedCheckButton("QuestTrackerCollapse", Container, Settings, "trackercollapse", "Green")
		Container.QuestTrackerCollapse:SetPoint("TOPLEFT", Container.ConfirmDelete, "BOTTOMLEFT", 0, -3)
		NS:AddCheckLabel(Container.QuestTrackerCollapse, L.QuestTrackerCollapse, "RIGHT", 15, 0)
		Container.QuestTrackerCollapse:SetToolTip(L.QuestTrackerCollapseTip)
		Container.QuestTrackerCollapse.PostUpdate = function(self) --, data, index)
			NS:GetOption("SetTrackerCollapse")()
			if self:GetChecked() then
--				ObjectiveTracker_Collapse()
				if WatchFrame_CollapseExpandButton_OnClick then
					securecall(WatchFrame_CollapseExpandButton_OnClick)
				elseif ObjectiveTracker_Collapse then
					securecall("ObjectiveTracker_Collapse")
				end
			end
		end
		CheckButtons[Container.QuestTrackerCollapse] = true -- true = use base settings, not branch
		if NS:IsValidVersion(WOW_PROJECT_MAINLINE) then
			Container.PetBattleHide = NS.Lib:CreateLedCheckButton("PetBattleHide", Container, Settings, "petbattlehide", "Green")
			Container.PetBattleHide:SetPoint("TOPLEFT", Container.QuestTrackerCollapse, "BOTTOMLEFT", 0, -3)
			NS:AddCheckLabel(Container.PetBattleHide, L.PetBattleHide, "RIGHT", 15, 0)
			Container.PetBattleHide:SetToolTip(L.PetBattleHideTip)
			CheckButtons[Container.PetBattleHide] = true -- true = use base settings, not branch
		end
	end

	Container.PortraitButton = NS:CreateDropList("PortraitButton", Container, Settings, "buttonicon", MMPortraits, "text", "value")
--	Container.PortraitButton:AddPadding(50)
	Container.PortraitButton:SetLabel(L.PortraitButton, "LEFT", 15, 0)
	Container.PortraitButton:SetToolTip(L.PortraitButtonTip)
	Container.PortraitButton:SetSize(100, 23)
	Container.PortraitButton:SetPoint("TOPRIGHT", Container, "TOPRIGHT", -15, -162)
	Controls[Container.PortraitButton] = true

	Container.OnLeftClick = NS:CreateDropList("OnLeftClick", Container, Settings, "LeftButton", OnLeftClick, "text", "value")
--	Container.OnLeftClick:AddPadding(25)
	Container.OnLeftClick:SetLabel(L.OnLeftClick, "LEFT", 15, 0)
	Container.OnLeftClick:SetToolTip(L.OnLeftClickTip)
	Container.OnLeftClick:SetSize(100, 23)
	Container.OnLeftClick:SetPoint("TOPLEFT", Container.PortraitButton, "BOTTOMLEFT", 0, -5)
	Container.OnLeftClick.PostUpdate = function(self) --, data, index)
		local Settings = NS:GetSettings("UI")
		if self.Data[self.Index] == 1 then
			Settings.RightButton = 2
		else
			Settings.RightButton = 1
		end
	end
	Controls[Container.OnLeftClick] = true

	Container.GoldOnly = NS:CreateDropList("GoldOnly", Container, Settings, "goldonly", GoldValues, "text", "value")
--	Container.GoldOnly:AddPadding(10)
	Container.GoldOnly:SetLabel(L.GoldOnly, "LEFT", 15, 0)
	Container.GoldOnly:SetToolTip(L.GoldOnlyTip)
	Container.GoldOnly:SetSize(100, 23)
	Container.GoldOnly:SetPoint("TOPLEFT", Container.OnLeftClick, "BOTTOMLEFT", 0, -5)
	Container.GoldOnly.PostUpdate = function(self)
		NS:GetOption("SetGoldOnly")()
	end
	Controls[Container.GoldOnly] = true

	local Availability = TalkingHeadFrame --select(5, GetAddOnInfo("Blizzard_TalkingHeadUI"))
	if Availability then --~= "MISSING" then
		Container.TalkingHead = NS:CreateDropList("TalkingHead", Container, NS:GetSettings("LOD").TalkingHeadFrame, "point", NS.TalkingHeadPos, "text", "value")
--		Container.TalkingHead:AddPadding(50)
		Container.TalkingHead:SetLabel(L.TalkingHead, "LEFT", 15, 0)
		Container.TalkingHead:SetToolTip(L.TalkingHeadTip)
		Container.TalkingHead:SetSize(100, 23)
		Container.TalkingHead:SetPoint("TOPLEFT", Container.GoldOnly, "BOTTOMLEFT", 0, -5)
		Container.TalkingHead.PostUpdate = function(self) --, data, index, key)
			local Entry = self.Table[self.Key]
			local Settings = NS:GetSettings("LOD").TalkingHeadFrame
			Settings.x = Entry and Entry.x
			Settings.y = Entry and Entry.y
			if TalkingHeadFrame then
				NS:GetOption("TalkingHeadPos")()
			end
		end
		Controls[Container.TalkingHead] = { table="LOD", [1]="TalkingHeadFrame" }
	end	

	Container.PortraitButton:AddDropHider(Container.OnLeftClick)
	Container.PortraitButton:AddDropHider(Container.GoldOnly)
	Container.OnLeftClick:AddDropHider(Container.GoldOnly)
--[[
	if Container.TalkingHead then
		Container.PortraitButton:AddDropHider(Container.TalkingHead)
		Container.OnLeftClick:AddDropHider(Container.TalkingHead)
		Container.GoldOnly:AddDropHider(Container.TalkingHead)
	end
]]--

--!!!!!!!!!!!!!!!!!!!!!!!!!!!! MAYBE NOT MOVE SOMEWHERE ELSE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ]]--
	local function MiniButtonExclude(object)
		local name = object:GetName()
		if not name or name == "" then return end
		if ObjectExclude[object] then return end
		for i=1, #MiniButtonExcludeList do
			if find(name, MiniButtonExcludeList[i]) then 
			return end
		end
		return name
	end
	local function MiniButtonFind(button)
		for tray=1, #TraySettings do
			for point, entry in pairs(TraySettings[tray]) do
				for i=1, #entry do
					if entry[i].button == button then
						return tray, point, entry, i, entry[i]
					end
				end		
			end
		end
	end
	local function ClearSaved(self)
		local Parent = self:GetParent()
		Parent.MiniButtonSave:SetEnabled(false)
		Parent.MiniButtonCancel:SetEnabled(false)
		Parent.MiniButtonFound:SetText("")
		local Find = Parent.MiniButtonFind
		Find.SelectMode = nil
		Find:SetChecked(false)
		Find:GetScript("OnClick")(Find)
		if CurrentButton.button then
			local f = CurrentButton.button
--			NS:GetOption("RestMiniButtonFuncs")(f)
			f:SetParent(CurrentButton.parent)
			f:SetScale(CurrentButton.scale)
			f:ClearAllPoints()
			f:SetPoint(CurrentButton.point, CurrentButton.relative, CurrentButton.topoint, CurrentButton.x, CurrentButton.y)
		end
		table.wipe(CurrentButton)
		NS:GetOption("ManageMiniButtons")()
		LoadTraySettings()
	end

	local SliderCheckInit
	local function SliderCheck()
		if Container.MiniButtonSave.DeleteMode then
			Container.MiniButtonSave:ResetIcon()
		end
	end
	
	Container.MiniButtonsLabel = NS:CreateLabel(Container, "UIMiniButtons", L.MiniButtons, 50)

	SetButton("MiniButtonFind", "Find", "Find")
	Container.MiniButtonFind = NS:CreateMenuButton(1, Container, "CheckButton", ButtonSettings, 22, true)
	Container.MiniButtonFind:SetSize(70, 18)
	Container.MiniButtonFind:HookScript("OnClick", function(self)
		local Parent = self:GetParent()
		if self:GetChecked() then
			Parent.MiniButtonFound:SetText("")
			Parent.MiniButtonCancel:SetEnabled(true)
			self:SetScript("OnUpdate", function(self)
				local obj = GetMouseFocus()
				if not obj or obj:GetObjectType() ~= "Button" then 
					return 
				end
				if  obj:GetParent() ~= Minimap then
					local name = obj:GetName()
					if obj == MiniMapTrackingButton or (obj.FauxParented and obj.FauxUnMovable) or (name and sub(name, 1, 4) == "DUF_") then
						return
					end
				end
				local text = MiniButtonExclude(obj)
				if text then
					if not self.SelectMode then
						self.SelectMode = true
						self:SetText(L.Select)
						self:SetIcon("Drop")
						Parent.MiniButtonSave:SetEnabled(false)
					end
					Parent.MiniButtonFound:SetText(text)
				end
			end)
		else
			self:SetText(L.Find)
			self:SetIcon("Find")
			self:SetScript("OnUpdate", nil)
			if self.SelectMode then
				self.SelectMode = nil
				Parent.MiniButtonSave:SetEnabled(true)
-- Set the SET button to enabled etc. update dummy settings and populate the tray with the new button
				local SetButton = Parent.MiniButtonFound:GetText()
				SetButton = SetButton and clip(SetButton)
				if SetButton == "" then
					return
				end
				local Tray, Point, Pointtable, Index, Entry = MiniButtonFind(SetButton)
				local Button = _G[SetButton]
				if not Entry then
					CurrentButton.button = Button
					local Point, Relative, ToPoint, X, Y = Button:GetPoint(1)
					CurrentButton.point = Point
					CurrentButton.relative = Relative
					CurrentButton.topoint = ToPoint
					CurrentButton.x = X
					CurrentButton.y = Y
					CurrentButton.parent = Button:GetParent()
					CurrentButton.scale = Button:GetScale()
					Tray = NS:GetOption("GetCurrentTray")():GetID()
					tinsert(TraySettings[Tray]["right"], { button=SetButton, offsetx=-120, offsety=0, scale=1 })
					Entry = TraySettings[Tray]["right"][#TraySettings[Tray]["right"]]
					NS:GetOption("PopulateTray")(Tray, TraySettings[Tray])
				else
					Parent.MiniButtonSave.Tray = Tray
					Parent.MiniButtonSave.Point = Point
					Parent.MiniButtonSave.Index = Index
					Parent.MiniButtonSave.DeleteMode = true
					Parent.MiniButtonSave:SetIcon("MenuDelete")
					Parent.MiniButtonSave:SetText(L.Remove)
					Parent.MiniButtonSave.ResetButton = Button
				end
				Parent.Tray = Tray
				NS.Lib.InitData(Parent.MiniButtonX, Entry, "offsetx")
				NS.Lib.InitData(Parent.MiniButtonY, Entry, "offsety")
				NS.Lib.InitData(Parent.MiniButtonScale, Entry, "scale")
--				Parent.MiniButtonX:SetEnabled(true)
				SliderCheckInit = true
			end
		end
	end)
	Container.MiniButtonFind:SetScript("OnHide", function(self)
		self.SelectMode = nil
		self:GetParent().MiniButtonFound:SetText("")
		self:SetChecked(false)
		self:GetScript("OnClick")(self)
	end)

	Container.MiniButtonFound = NS.Lib:CreateFont("MiniButtonFound", Container, "", "CENTER", "CENTER")
	Container.MiniButtonFound:SetTextColor(1, 1, 0)
	Container.MiniButtonFound:SetPoint("TOP", Container.MiniButtonsLabel, "BOTTOM", 0, -8)
	

	SetButton("MiniButtonSave", "Save", "Save")
	Container.MiniButtonSave = NS:CreateMenuButton(1, Container, "Button", ButtonSettings, 22, true)
	Container.MiniButtonSave:SetSize(70, 18)
	Container.MiniButtonSave:HookScript("OnClick", function(self)
		if self.DeleteMode then
			TraySettings[self.Tray][self.Point][self.Index] = nil
			NS:GetOption("ResetMiniButtonPos")(self.ResetButton)
		end
		local Settings = NS:GetSettings("UI").buttontrays
		table.wipe(Settings)
		NS.Lib:CopyTable(TraySettings, Settings)
		ClearSaved(self)
	end)
	hooksecurefunc(Container.MiniButtonSave, "SetEnabled", function(self)
		self:ResetIcon()
	end)
	function Container.MiniButtonSave:ResetIcon()
		self.DeleteMode = nil
		self:SetIcon("Save")
		self:SetText(L.Save)
		self.Tray = nil
		self.Point = nil
		self.Index = nil
		self.ResetButton = nil
		SliderCheckInit = nil
	end
	hooksecurefunc(Container.MiniButtonSave, "SetEnabled", function(self, enabled)
		local parent = self:GetParent()
		parent.MiniButtonX:SetEnabled(enabled)
		parent.MiniButtonY:SetEnabled(enabled)
		parent.MiniButtonScale:SetEnabled(enabled)
	end)

	SetButton("MiniButtonCancel", "Cancel", "Cancel")
	Container.MiniButtonCancel = NS:CreateMenuButton(1, Container, "Button", ButtonSettings, 22, true)
	Container.MiniButtonCancel:SetSize(70, 18)
	Container.MiniButtonCancel:SetEnabled(false)
	Container.MiniButtonCancel:SetScript("OnHide", function(self)
		ClearSaved(self)
	end)
	Container.MiniButtonCancel:HookScript("OnClick", function(self)
		ClearSaved(self)
	end)

--	Container.MiniButtonCancel:SetPoint("BOTTOMLEFT", Container, 0, -3)
--	Container.MiniButtonSave:SetPoint("BOTTOM", Container.MiniButtonCancel, "TOP", 0, 3)
--	Container.MiniButtonFind:SetPoint("BOTTOM", Container.MiniButtonSave, "TOP", 0, 3)
--	Container.MiniButtonsLabel:SetPoint("BOTTOM", Container, 0,(Container.MiniButtonCancel:GetHeight() * 3) + 6)
	Container.MiniButtonsLabel:SetPoint("TOP", Container.Skins, "BOTTOMRIGHT", 70, -25)
	Container.MiniButtonFind:SetPoint("TOPLEFT", Container.Skins, "BOTTOMLEFT", 5, -20)
	Container.MiniButtonSave:SetPoint("TOP", Container.MiniButtonFind, "BOTTOM", 0, -3)
	Container.MiniButtonCancel:SetPoint("TOP", Container.MiniButtonSave, "BOTTOM", 0, -3)

	local function SliderInit(self)
		if SliderCheckInit then
			SliderCheck()
		end
		NS:GetOption("PopulateTray")(NS:GetOption("GetCurrentTray")():GetID(), TraySettings[self:GetParent().Tray])
	end

	local Anchor = QueueStatusMinimapButton or QueueStatusButton 
	Anchor = Anchor or MailFrame
	local Xmin = floor((NS:GetFrame("Utility", "MiniButton"):GetLeft()-Anchor:GetRight())+80)--32)
	
	if QueueStatusMinimapButton or Xmin > 0 then --?????
		Xmin = -Xmin
	end
	local DummySettings = { offsetx=0, offsety=0, scale=1 }
	Container.MiniButtonX = NS.Lib:CreateSlider("MBX", Container, DummySettings, "offsetx", Xmin, true, 0, true, 1, 90, 20)
	Container.MiniButtonX:SetLabel("X", "TOP", 0, -2)
	Container.MiniButtonX:SetPoint("BOTTOMLEFT", Container.MiniButtonSave, "BOTTOMRIGHT", 10, -3)
	function Container.MiniButtonX:PostUpdate()
		SliderInit(self)
	end

	Container.MiniButtonY = NS.Lib:CreateSlider("MBY", Container, DummySettings, "offsety", -10, true, 10, true, 1, 90, 20)
	Container.MiniButtonY:SetLabel("Y", "TOP", 0, -2)
	Container.MiniButtonY:SetPoint("LEFT", Container.MiniButtonX, "RIGHT", 10, 0)
	function Container.MiniButtonY:PostUpdate()
		SliderInit(self)
	end

	Container.MiniButtonScale = NS.Lib:CreateSlider("MBScale", Container, DummySettings, "scale", 50, true, 150, true, 1, 90, 20, nil, nil, 100)
	Container.MiniButtonScale:SetLabel(L.Scale, "TOP", 0, -2)
	Container.MiniButtonScale:SetPoint("LEFT", Container.MiniButtonY, "RIGHT", 10, 0)
	function Container.MiniButtonScale:PostUpdate()
		SliderInit(self)
	end

	Container.MiniButtonSave:SetEnabled(false)
--	Container.MiniButtonX:SetEnabled(false)

--[[!!!!!!!!!!!!!!!!!!!!!!!!!!!! END MOVE SOMEWHERE ELSE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ]]--
	local x = Container.MiniButtonScale:GetWidth()*3 + Container.MiniButtonCancel:GetWidth() + 26
--	Container:SetSize(x+20, Container.Skins:GetHeight() + YPadding)
	Container:HookScript("OnShow", function(self)
		NS:PlaceCopyButton(CopySettings, 30, -15, "RIGHT", Container:GetParent().Close, "LEFT", -15, 0, Container)
	end)
	Container:GetScript("OnShow")(Container)
--	Container:Show()
	return Container
end

NS:RegisterModule("UI", "UI", "UI", ConfigureUI)
--[[ Might add later??
	local AbAction = { actionbuttonclick = GetCVarBool("ActionButtonUseKeyDown") }
	Container.ActionClick = NS.Lib:CreateLedCheckButton("ActionClick", Container, AbAction, "actionbuttonclick", "Green")
	Container.ActionClick:SetPoint("TOPLEFT", Container.Other, "BOTTOMLEFT", 0, -5)
	Container.ActionClick:SetLabel(L.ActionButton, "RIGHT")
--	Container.ActionClick:SetToolTip(L.ActionClickTip)
--	function Container.ActionClick:SetLabel()
--		self.Label:SetText(self.Data[self.Index] == true and L.GetText(L.ActionButton, L.Down) or L.GetText(L.ActionButton, L.Up))
--	end
--	Container.ActionClick:SetLabel()
	function Container.ActionClick.PostUpdate(self)
		SetCVar("ActionButtonUseKeyDown", self.Data[self.Index] == true and "1" or "0")
--		self:SetLabel()
	end

]]--


